*
*  PERF16.PRG
*  SEEK/SCAN WHILE vs SCAN FOR 
*
*  (c) Copyright 1994   Prosoft Systems, Inc.
*           Prosoft Systems, Inc.
*           10560 Main St., Suite 418
*           Fairfax, VA  22030
*           703-591-0101 voice
*           703-591-0108 fax
*  Author:  Drew Speedie
*
clear all
clear
set talk off
set exclusive off
set safety off 

workdir = "C:\FPWWORK"

********************************************************
*  records in PERFBAL are in order of data-entry
*  ba_state="CA" records are scattered throughout
********************************************************

********************************************************
*  SCAN FOR with the ba_state index tag set:
********************************************************
use PERFBAL order tag ba_state
start = seconds()
FOR xx = 1 to 100
  SCAN FOR ba_state = "CA"
    yy = xx
  ENDSCAN
ENDFOR
@ 1,1 say "SCAN FOR/order: " + str(seconds()-start,7,3) ;
     font "Arial",30 style "B" ;
     color rgb(128,0,0,255,255,255)

********************************************************
*  SCAN FOR with no index tag order set:
********************************************************
close databases
use PERFBAL order tag ba_state
set order to 0
start = seconds()
FOR xx = 1 to 100
  SCAN FOR ba_state = "CA"
    yy = xx
  ENDSCAN
ENDFOR
@ 4,1 say "SCAN FOR/no order: " + str(seconds()-start,7,3) ;
     font "Arial",30 style "B" ;
     color rgb(128,0,0,255,255,255)

********************************************************
*  SEEK/SCAN WHILE: 
********************************************************
close databases
use PERFBAL order tag ba_state
start = seconds()
FOR xx = 1 to 100
  seek "CA"
  SCAN WHILE ba_state = "CA"
    yy = xx
  ENDSCAN
ENDFOR
@ 7,1 say "SEEK/SCAN WHILE: " + str(seconds()-start,7,3) ;
     font "Arial",30 style "B" ;
     color rgb(0,0,128,255,255,255)

********************************************************
*  SEEK/DO WHILE: 
********************************************************
close databases
use PERFBAL order tag ba_state
start = seconds()
FOR xx = 1 to 100
  seek "CA"
  DO WHILE ba_state = "CA"
    yy = xx
    skip
  ENDDO 
ENDFOR
@ 10,1 say "SEEK/DO WHILE: " + str(seconds()-start,7,3) ;
     font "Arial",30 style "B" ;
     color rgb(0,128,0,255,255,255)

********************************************************
*  create a new version of PERFBAL that contains records
*  physically ordered by ba_state 
********************************************************
close databases
use PERFBAL
start = seconds()
sort on ba_state to (workdir+"\PERF16")
use (workdir+"\PERF16")
sorttime = seconds()-start

********************************************************
*  SCAN FOR with the ba_state index tag set:
********************************************************
start = seconds()
index on ba_state tag ba_state 
indextime = seconds()-start
start = seconds()
FOR xx = 1 to 100
  SCAN FOR ba_state = "CA"
    yy = xx
  ENDSCAN
ENDFOR
@ 13,1 say "SCAN FOR/order: " + str(seconds()-start,7,3) ;
     font "Arial",30 style "B" ;
     color rgb(128,0,0,255,255,255)
********************************************************
*  times for SORTing and INDEXing the new table --
*  doing either before the SCAN FOR would still be
*  faster than the SCAN FOR with an ORDER set
********************************************************
@ 13.5,col()+2 say alltrim(str(sorttime,5,2)) + space(2) + alltrim(str(indextime,5,2)) ; 
     font "Arial",20 style "B" ;
     color rgb(128,0,0,255,255,255)

********************************************************
*  SCAN FOR with no index tag order set:
********************************************************
close databases
use (workdir+"\PERF16")
set order to 0
start = seconds()
FOR xx = 1 to 100
  SCAN FOR ba_state = "CA"
    yy = xx
  ENDSCAN
ENDFOR
@ 16,1 say "SCAN FOR/no order: " + str(seconds()-start,7,3) ;
     font "Arial",30 style "B" ;
     color rgb(128,0,0,255,255,255)

********************************************************
*  SEEK/SCAN WHILE: 
********************************************************
close databases
use (workdir+"\PERF16")
set order to tag ba_state
start = seconds()
FOR xx = 1 to 100
  seek "CA"
  SCAN WHILE ba_state = "CA"
    yy = xx
  ENDSCAN
ENDFOR
@ 19,1 say "SEEK/SCAN WHILE: " + str(seconds()-start,7,3) ;
     font "Arial",30 style "B" ;
     color rgb(0,0,128,255,255,255)

close databases
use PERFBAL
start = seconds()
index on ba_state to (workdir+"\statetag") FOR ba_state = "CA"
indextime = seconds()-start
FOR xx = 1 to 100
  SCAN  
    yy = xx
  ENDSCAN
ENDFOR
xx = seconds()
set index to 
erase (workdir+"\statetag.idx")
indextime = indextime + (seconds()-xx)
@ 22,1 say "SCAN/temp idx: " + str(seconds()-start,7,3) ;
     font "Arial",30 style "B" ;
     color rgb(128,0,128,255,255,255)
@ 22.5,col()+2 say alltrim(str(indextime,5,2)) ; 
     font "Arial",20 style "B" ;
     color rgb(128,0,128,255,255,255)

erase (workdir+"\perf16.dbf")
erase (workdir+"\perf16.cdx")
clear all
return